-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-- Written by Kimberly L. Tripp, SQLskills.com
-- All Rights Reserved.
-- 
-- For more scripts, sample code and additional training resources,
-- go to http://www.SQLskills.com
--	Immerse yourself in SQL Server
--
-- Disclaimer - Thoroughly test this script, execute at your own risk.
-- ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

-- Use a "TestDB" Database with at Least 3-8 MB Free Space.

USE TestFGDB
go

SET STATISTICS IO OFF
SET NOCOUNT ON
go
IF OBJECTPROPERTY(object_id('dbo.DemoTableHeap'), 'IsUserTable') = 1
	DROP TABLE dbo.DemoTableHeap
go
CREATE TABLE dbo.DemoTableHeap
(
	col1	int	 		identity(100,10),
	col2	datetime 	CONSTRAINT DemoTableHeapCol2Default 
							DEFAULT current_timestamp,
	col3	datetime 	CONSTRAINT DemoTableHeapCol3Default 
							DEFAULT getdate(),
	col4	char(30) 	CONSTRAINT DemoTableHeapCol4Default 
							DEFAULT suser_name(),
	col5	char(30)  	CONSTRAINT DemoTableHeapCol5Default 
							DEFAULT user_name(),
	col6	char(100)  	CONSTRAINT DemoTableHeapCol6Default 
							DEFAULT 'Long text value of "Now is the time for all good men to come to the aid of their country."',
	col7	varchar(200)  	CONSTRAINT DemoTableHeapCol7Default 
							DEFAULT 'Narrow value.'
) ON [FG]
go
DECLARE @EndTime	datetime
SELECT @EndTime = dateadd(ss, 35, getdate())
WHILE getdate() <= @EndTime
BEGIN
	INSERT dbo.DemoTableHeap DEFAULT VALUES
END
EXEC sp_spaceused 'dbo.DemoTableHeap'
go
-- Record the RowCount = <rows>
-- Figure out the number of pages by dividing "data" by 8
-- SELECT <data>/8 = <pages>
-- This SHOULD MATCH the I/Os shown by SELECT COUNT(*)

SET STATISTICS IO ON
go
SELECT COUNT(*) AS [RowCount] 
FROM dbo.DemoTableHeap
go

-- Modify roughly 15% - keep track of the rows affected
UPDATE dbo.DemotableHeap
	SET col7 = 'This is a test to create some fragmentation. The previously small column is now filled to capacity. This is a test to create some fragmentation. The previously small column is now filled to capacity.'
	WHERE col1 % 7 = 0
SELECT @@rowcount AS [Rows Modified]
go

-- Keep track of Rows Affected Here = <rows>

-- Check the space again and calculate # of Pages
SET STATISTICS IO OFF
go
EXEC sp_spaceused 'dbo.DemoTableHeap', true
go

-- Figure out the number of pages by dividing "data" by 8
-- SELECT <data>/8 = <pages>
-- This SHOULD MATCH the I/Os shown by SELECT COUNT(*)

-- Does this match the I/Os shown by SELECT COUNT(*)
-- ??
SET STATISTICS IO ON
go
SELECT COUNT(*) AS [RowCount] 
FROM dbo.DemoTableHeap
go

-- What does is equal?
-- Pages + Rows Modified that caused Forwarding Pointers
-- Do the math Total I/Os - Pages in Table = ???
-- SELECT <IOs> - <pages> = ????
-- So what does that number represent?



-- Forwarding Pointers!
-- A Forwarding Pointer is ALWAYS honored. This is required for locks
-- to actually read the row resource (and release) and this actually
-- makes sense for most operations and the fact that if rows move
-- we could end up not reading a row we were meant to read...
-- However, it is very expensive for a table scan.